use std::{time::{Instant, Duration}, thread};

use pancurses::{initscr, noecho, curs_set, cbreak, endwin, Window};

const DATA: &str = include_str!("car.txt");
const EXTRA: i32 = 5;

fn main() {
    // want it to keep running even if this fails
    let _ = ctrlc::set_handler(||());

    let longest_line = DATA.lines().map(<str>::len).max().unwrap_or_default();

    let data = DATA.split("\n\n").cycle();

    let window = initscr();
    
    noecho();
    curs_set(0);
    cbreak();
    window.nodelay(true);

    let mut x = window.get_max_x() + EXTRA;
    let base_y = window.get_max_y() / 2;

    let mut next_tick = Instant::now();

    for frame in data {
        next_tick += Duration::from_millis(50);
        
        x -= 1;

        if x < -(longest_line as i32 + EXTRA) {
            break;
        }

        window.clear();
        print(&window, base_y - frame.lines().count() as i32 / 2, x, frame);
        window.refresh();

        thread::sleep(next_tick - Instant::now());
    }

    endwin();
}

/// prints to the window, multiline works, cutoff at the edge works
/// 
/// assumes 1 bytes chars
fn print<T: AsRef<str>>(window: &Window, mut y: i32, x: i32, data: T) {
    let data = data.as_ref();

    let cols = window.get_max_x();

    for line in data.lines() {
        if x < 0 { // off the left edge
            window.mvprintw(y, 0, &line.get(-x as usize..).unwrap_or(""));
        } else if x > cols {
            // extra case to prevent subtraction with overflow below
        } else if x + line.len() as i32 > cols { // off the right edge
            window.mvprintw(y, x, &line[..cols as usize - x as usize]);
        } else { // normal
            window.mvprintw(y, x, line);
        }
        y += 1;
    }
}